// lcd
#include <LiquidCrystal.h>

// SD card
#include <SD.h>
#include <SPI.h>

#define RED_LED 5
#define GREEN_LED 6

int green_brightness = 0;
int red_brightness = 0;
const int fade = 5;
const int led_threshold = 255;

#define RED_BUTTON A2
#define BLUE_BUTTON A1

#define BUZZER A5
#define MQ3 A0

const char FILE_NAME[] = "data.txt";
const int CS_PIN = 10;
File file;

#define D4 A3
const int RS = 10, EN = 9, D5 = 4, D6 = 3, D7 = 2;
LiquidCrystal lcd(RS, EN, D4, D5, D6, D7);

// 5s measure time
const int measureTime = 5000;
const int ALCOHOL_THRESHOLD = 120;

int sensorValue;
int red_button_state = 0;
int blue_button_state = 0;
int offset = 0;
void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  Serial.println("start of serial");
  // LEDs
  pinMode(RED_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);

  // Buttons
  pinMode(RED_BUTTON, INPUT);
  pinMode(BLUE_BUTTON, INPUT);

  // Buzzer
  pinMode(BUZZER, OUTPUT);

  // lcd
  lcd.begin(16, 2);

  // SD card
  initializeSD();
  createFile(FILE_NAME);
  closeFile();
}

void loop() {
  lcd.setCursor(0, 0);

  Serial.println(analogRead(MQ3));

  // state of each button
  delay(200);
  red_button_state = digitalRead(RED_BUTTON);
  blue_button_state = digitalRead(BLUE_BUTTON);

  if (red_button_state == HIGH) {
    lcd.clear();
    Serial.println("[HOME] Red button pressed");
    openFile(FILE_NAME, FILE_READ);
    for (int i = 0; i < offset; i++) {
      readLine();
    }
    lcd.print(readLine());
    offset++;
    closeFile();
  } else if (blue_button_state == HIGH) {
    Serial.println("[HOME] Blue button pressed");
    offset = 0;
    measureAlcohol();
  }
}

void measureAlcohol() {
  Serial.println("[ALC] entering measureAlcohol");

  delay(500);
  // short buzz, marking start of measurement proccess
  digitalWrite(BUZZER, HIGH);
  delay(100);
  digitalWrite(BUZZER, LOW);

  int maxDetected = 0;
  int timeElapsed = 0;
  int incrementingUnit = 50;
  while (timeElapsed < measureTime) {
    Serial.println("[ALC] measuring...");
    int reading = analogRead(MQ3);
    if (reading > maxDetected) {
      maxDetected = reading;
    }

    delay(incrementingUnit);
    timeElapsed += incrementingUnit;
  }

  Serial.println("[ALC] buzzer!!!");
  // short buzz, marking end of measurement proccess
  digitalWrite(BUZZER, HIGH);
  delay(100);
  digitalWrite(BUZZER, LOW);

  lcd.setCursor(0, 0);
  lcd.print(maxDetected);
  Serial.print("[ALC] printing to display measured alcohol");
  Serial.println(maxDetected);

  Serial.println("[ALC] Starting corresponding LED");
  // red LED -> over limit
  // green LED -> under limit
  if (maxDetected > ALCOHOL_THRESHOLD) {
    // digitalWrite(RED_LED, HIGH);
    while (red_brightness < led_threshold) {
      analogWrite(RED_LED, red_brightness);
      red_brightness += fade;
      delay(30);
    }
  } else {
    // digitalWrite(GREEN_LED, HIGH);
    while (green_brightness < led_threshold) {
      analogWrite(GREEN_LED, green_brightness);
      green_brightness += fade;
      delay(30);
    }
  }

  lcd.setCursor(0, 1);
  lcd.println("continue - save");
  lcd.setCursor(0, 0);
  Serial.println("[ALC] waiting for user to press button");


  while (1) {
    red_button_state = digitalRead(RED_BUTTON);
    blue_button_state = digitalRead(BLUE_BUTTON);
    delay(100);
    if (red_button_state == HIGH) {
      Serial.println("[ALC] Red button, saving current measure");
      openFile(FILE_NAME, FILE_WRITE);
      writeToFile(maxDetected);
      closeFile();
      break;
    } else if (blue_button_state == HIGH) {
      Serial.println("[ALC] Blue button, going back to home menu");
      break;
    }
  }

  Serial.println("[ALC] turning off LEDs");
  while (red_brightness >= 0) {
    analogWrite(RED_LED, red_brightness);
    red_brightness -= fade;
    delay(30);
  }
  while (green_brightness >= 0) {
    analogWrite(GREEN_LED, green_brightness);
    green_brightness -= fade;
    delay(30);
  }
  lcd.clear();
}

void initializeSD() {
  Serial.println("Initializing SD card...");
  pinMode(CS_PIN, OUTPUT);

  if (SD.begin()) {
    Serial.println("SD card is ready to use.");
  } else {
    Serial.println("SD card initialization failed");
    return;
  }
}

int createFile(char filename[]) {
  file = SD.open(filename, FILE_WRITE);

  if (file) {
    Serial.println("File created successfully.");
    return 1;
  } else {
    Serial.println("Error while creating file.");
    return 0;
  }
}

int writeToFile(int num) {
  if (num) {
    file.println(num), DEC;

    Serial.println("Writing to file: ");
    Serial.println(num);

    return 1;
  } else {
    Serial.println("Couldn't write to file");
    return 0;
  }
}

void openFile(char filename[], char mode[]) {
  file = SD.open(filename, mode);
  if (file) {
    Serial.println("File opened with success!");
    return 1;
  } else {
    Serial.println("Error opening file...");
    return 0;
  }
}

void closeFile() {
  if (file) {
    file.close();
    Serial.println("File closed");
  }
}

String readLine() {
  String received = "";
  char ch;
  while (file.available()) {
    ch = file.read();

    if (ch == '\n') {
      return String(received);
    } else {
      received += ch;
    }
  }
  return "";
}
